@@ -5,8 +5,9 @@ module Github.Repos (
 
 -- * Querying repositories
  userRepos
-,organizationRepos
 ,userRepo
+,organizationRepos
+,organizationRepo
 ,contributors
 ,contributorsWithAnonymous
 ,languagesFor
@@ -17,9 +18,8 @@ module Github.Repos (
 
 -- * Modifying repositories
 -- |
--- Only authenticated users may modify repositories.  Currently only
--- /HTTP basic access authentication/ is implemented.
-,BasicAuth
+-- Only authenticated users may modify repositories.
+,GithubAuth(..)
 
 -- ** Create
 ,createRepo
@@ -43,6 +43,7 @@ import Github.Private
 import Network.HTTP.Conduit
 import qualified Data.ByteString.Char8 as BS
 import Control.Applicative
+import qualified Control.Exception as E
 import Network.HTTP.Types
 
 -- | Filter the list of the user's repos using any of these constructors.
@@ -77,6 +78,12 @@ userRepos userName Private =
 organizationRepos :: String -> IO (Either Error [Repo])
 organizationRepos orgName = githubGet ["orgs", orgName, "repos"]
 
+-- | A specific organization repo, by the organization name.
+--
+-- > organizationRepo "thoughtbot" "github"
+organizationRepo :: String -> String -> IO (Either Error Repo)
+organizationRepo orgName repoName = githubGet ["orgs", orgName, repoName]
+
 -- | Details on a specific repo, given the owner and repo name.
 --
 -- > userRepo "mike-burns" "github"
@@ -133,6 +140,7 @@ data NewRepo = NewRepo {
 , newRepoHasIssues    :: (Maybe Bool)
 , newRepoHasWiki      :: (Maybe Bool)
 , newRepoHasDownloads :: (Maybe Bool)
+, newRepoAutoInit     :: (Maybe Bool)
 } deriving Show
 
 instance ToJSON  NewRepo where
@@ -143,6 +151,7 @@ instance ToJSON  NewRepo where
                   , newRepoHasIssues    = hasIssues
                   , newRepoHasWiki      = hasWiki
                   , newRepoHasDownloads = hasDownloads
+                  , newRepoAutoInit     = autoInit
                   }) = object
                   [ "name"                .= name
                   , "description"         .= description
@@ -150,24 +159,24 @@ instance ToJSON  NewRepo where
                   , "private"             .= private
                   , "has_issues"          .= hasIssues
                   , "has_wiki"            .= hasWiki
-                  , "has_downloads"       .= hasDownloads
+                  , "auto_init"           .= autoInit
                   ]
 
 newRepo :: String -> NewRepo
-newRepo name = NewRepo name Nothing Nothing Nothing Nothing Nothing Nothing
+newRepo name = NewRepo name Nothing Nothing Nothing Nothing Nothing Nothing Nothing
 
 -- |
 -- Create a new repository.
 --
--- > createRepo (user, password) (newRepo "some_repo") {newRepoHasIssues = Just False}
-createRepo :: BasicAuth -> NewRepo -> IO (Either Error Repo)
+-- > createRepo (GithubUser (user, password)) (newRepo "some_repo") {newRepoHasIssues = Just False}
+createRepo :: GithubAuth -> NewRepo -> IO (Either Error Repo)
 createRepo auth = githubPost auth ["user", "repos"]
 
 -- |
 -- Create a new repository for an organization.
 --
--- > createOrganizationRepo (user, password) "thoughtbot" (newRepo "some_repo") {newRepoHasIssues = Just False}
-createOrganizationRepo :: BasicAuth -> String -> NewRepo -> IO (Either Error Repo)
+-- > createOrganizationRepo (GithubUser (user, password)) "thoughtbot" (newRepo "some_repo") {newRepoHasIssues = Just False}
+createOrganizationRepo :: GithubAuth -> String -> NewRepo -> IO (Either Error Repo)
 createOrganizationRepo auth org = githubPost auth ["orgs", org, "repos"]
 
 data Edit = Edit {
@@ -204,8 +213,8 @@ instance ToJSON  Edit where
 -- |
 -- Edit an existing repository.
 --
--- > editRepo (user, password) "some_user" "some_repo" def {editDescription = Just "some description"}
-editRepo :: BasicAuth
+-- > editRepo (GithubUser (user, password)) "some_user" "some_repo" def {editDescription = Just "some description"}
+editRepo :: GithubAuth
      -> String      -- ^ owner
      -> String      -- ^ repository name
      -> Edit
@@ -218,30 +227,25 @@ editRepo auth user repo body = githubPatch auth ["repos", user, repo] b
 -- |
 -- Delete an existing repository.
 --
--- > deleteRepo (user, password) "thoughtbot" "some_repo"
-deleteRepo :: BasicAuth
+-- > deleteRepo (GithubUser (user, password)) "thoughtbot" "some_repo"
+deleteRepo :: GithubAuth
            -> String      -- ^ owner
            -> String      -- ^ repository name
            -> IO (Either Error ())
 deleteRepo auth owner repo = do
-  requestToken >>= either (return . Left) (sendToken)
+  result <- doHttps "DELETE" url (Just auth) Nothing
+  case result of
+      Left e -> return (Left (HTTPConnectionError e))
+      Right resp ->
+          let status = responseStatus resp
+              headers = responseHeaders resp
+          in if status == notFound404
+                -- doHttps silently absorbs 404 errors, but for this operation
+                -- we want the user to know if they've tried to delete a
+                -- non-existent repository
+             then return (Left (HTTPConnectionError
+                                (E.toException
+                                 (StatusCodeException status headers))))
+             else return (Right ())
   where
-    -- APIv3 does not support deletion, so we use APIv2 for that
-    url = "https://github.com/api/v2/json/repos/delete/" ++ owner ++ "/" ++ repo
-
-    requestToken :: IO (Either Error DeleteToken)
-    requestToken = githubAPI "POST" url (Just auth) (Nothing :: Maybe Value)
-
-    sendToken (DeleteToken t) = do
-      let body = RequestBodyBS $ renderSimpleQuery False [("delete_token", t)]
-      result <- doHttps "POST" url (Just auth) (Just body)
-      return $ either (Left . HTTPConnectionError)
-                      (const $ Right ())
-                      result
-
-newtype DeleteToken = DeleteToken BS.ByteString
-  deriving Show
-
-instance FromJSON DeleteToken where
-  parseJSON (Object o) = DeleteToken <$> o .: "delete_token"
-  parseJSON _          = fail "Could not build a DeleteToken"
+    url = "https://api.github.com/repos/" ++ owner ++ "/" ++ repo
